
Bird = {};
mt_bird = {};
mt_bird.__index = Bird;
bird_objects = {};


function Bird:new(aCppPointer)
   return setmetatable({cppPointer = aCppPointer},mt_bird);
end;
---------------------------------------------------------------------------

---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------


---- "Instance" variables ----------------------------
-- Each Bird will have one instance of each of these variables
-- so you can think of this file as representing the Bird as a class with the instance variables
-- and methods shown below



--  Bird state constants
	
-- The function "InitializeBeforeWaveStart" is called before each wave is started
-- or re-started
-- The function "Update" is called every game cycle


---- Functions --------------------------------------------------------------------------------

	
-----------------------------------------------------------------------------------------------------
------------------------ Required functions ---------------------------------------------------------
-----------------------------------------------------------------------------------------------------
function Bird:InitializeGameObject(  )
	self.scale = 1;
	SetLuaGameObjectScale( self.cppPointer, self.scale );
	self.bitmapWidth, self.bitmapHeight = GetLuaGameObjectBitmapSize( self.cppPointer );
	self.collisionBoxWidth = self.bitmapWidth * self.scale - 5;
	self.collisionBoxHeight = self.bitmapHeight * self.scale - 5;
	SetLuaGameObjectCollisionBoxInfo( self.cppPointer, self.collisionBoxWidth, self.collisionBoxHeight, 0 );
	self.playfieldMinX, self.playfieldMaxX, self.playfieldMinY, self.playfieldMaxY = GetPlayfieldLimits();

	
   self.BIRD_SPEED = 2;


	self.PURSUING = 1;
   self.RETREATING = 2;
   self.TRACING_WALL = 3;
	self.NULL_STATE = -1;
   

	self.current_state = self.NULL_STATE;

	self.shot_timer = 75;

	self.destination_x = 0;
	self.destination_y = 0;
	
	self.playerX = 0;
	self.playerY = 0;

   self.PURUIT_TRACK_WAIT_TIME = 50;
   self.pursuit_tracking_timer = self.PURUIT_TRACK_WAIT_TIME;


   self.PURSUIT_TIME = 150;
   self.pursuit_timer = self.PURSUIT_TIME;

   self.RETREAT_TIME =120;
   self.retreat_timer = self.RETREAT_TIME;

   self.tracing_wall = nil;
   self.trace_direction = nil;

   self.WALL_TRACE_TIME = 100;
   self.wall_trace_timer = self.WALL_TRACE_TIME;

end

-----------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------
function Bird:TrackPlayer()
--   LogLevel1("Entering Bird:TrackPlayer()");
	local howFarAway = 50;

	local degrees = GenerateRandomNumber(0,360);

	local playerCenterX = self.playerX + (self.collisionBoxWidth / 2);
	local playerCenterY = self.playerY + (self.collisionBoxHeight / 2);

	local radians = degrees * (3.141519 / 180);

	local tempX = math.sin(radians) * howFarAway;
	local tempY = math.cos(radians) * howFarAway;

	tempX = tempX + playerCenterX;
	tempY = tempY + playerCenterY;

	self.destination_x = tempX;
	self.destination_y = tempY;

	self.current_state = self.PURSUING;
--   LogLevel1("Leaving Bird:TrackPlayer()");      
end;
   
-----------------------------------------------------------------------------------------------------
   
function Bird:CreateShot()

	self.playerX, self.playerY = GetPlayerXAndY();


	local xv = GenerateRandomNumber(2,5);
	local yv = GenerateRandomNumber(2,5);

	if( self.playerX <= self.x ) then
		xv = -xv;
	end;

	if( self.playerY <= self.y ) then
		yv = -yv;
	end;


	local addedBirdShot = AddGameObject(3, self.x + 10, self.y +10 );
	SetLuaGameObjectXVAndYV( addedBirdShot, xv,yv );


end;


-----------------------------------------------------------------------------------------------------

function Bird:StartRetreating()
--   LogLevel1("Entering Bird:StartRetreating()");
	self.current_state = self.RETREATING;
--   LogLevel1("Leaving Bird:StartRetreating()");
end;

-----------------------------------------------------------------------------------------------------

function Bird:StartTracing(dx,dy)
--   LogLevel1("Entering Bird:StartTracing()");
	self.current_state = self.TRACING_WALL;
   self.wall_trace_timer = self.WALL_TRACE_TIME;
   if( WasCollisionWithLeftWall(self) ) then
      self.tracing_wall = WEST;
      if(dy > 0 ) then
         self.trace_direction = SOUTH;
      else
         self.trace_direction = NORTH;
      end;

   elseif( WasCollisionWithRightWall( self )) then
      self.tracing_wall = EAST;
      if(dy > 0 ) then
         self.trace_direction = SOUTH;
      else
         self.trace_direction = NORTH;
      end;

   elseif(WasCollisionWithUpperWall(self)) then
      self.tracing_wall = NORTH;
      if(dx > 0 ) then
         self.trace_direction = EAST;
      else
         self.trace_direction = WEST;
      end;
   elseif(WasCollisionWithLowerWall(self)) then
      self.tracing_wall = SOUTH;  
      if(dx > 0 ) then
         self.trace_direction = EAST;
      else
         self.trace_direction = WEST;
      end;

   elseif(true) then
      LogLevel1("Bird:StartTracing: Error, collision wall not detected");
   end;

--   LogLevel1("Bird:StartTracing.  self.tracing_wall = " .. self.tracing_wall );

--   LogLevel1("Leaving Bird:StartTracing()");
end;


-----------------------------------------------------------------------------------------------------

function Bird:HandlePursuing()

   self.pursuit_timer = self.pursuit_timer - 1;
   if( self.pursuit_timer <= 0 ) then
      self.pursuit_timer = self.PURSUIT_TIME;
      local chanceToRetreat = GenerateRandomNumber(1,100);
      if( chanceToRetreat < 20) then
         self:StartRetreating();
         return;
      end;
   end;

   self.pursuit_tracking_timer = self.pursuit_tracking_timer  - 1;
   if(self.pursuit_tracking_timer <= 0 ) then
      self.pursuit_tracking_timer = self.PURUIT_TRACK_WAIT_TIME;
      self:TrackPlayer();
   end;


   local dx = 0;
   local dy = 0;

   local bufferDistance = 5;
    
	if( self.x > self.destination_x + bufferDistance) then      
		dx = - self.BIRD_SPEED;
	end;

	if( self.x < self.destination_x  - bufferDistance) then
		 dx =self.BIRD_SPEED ;
	end;

	if( self.y > self.destination_y + bufferDistance ) then
		 dy = - self.BIRD_SPEED;
	end;

	if( self.y < self.destination_y - bufferDistance) then
		 dy = self.BIRD_SPEED;
	end;

	self.x = self.x + dx;
   self.y = self.y + dy;

	if ( WasCollisionWithLeftRightWall(self) ) then
      local changeToTraceWall = GenerateRandomNumber(1,100);
      if(changeToTraceWall < 20 ) then
         self:StartTracing(dx,dy);
         return;
      end;
      self.x = self.x -dx;
	end;

	if ( WasCollisionWithUpDownWall(self) ) then
      local changeToTraceWall = GenerateRandomNumber(1,100);
      if(changeToTraceWall < 20 ) then
         self:StartTracing(dx,dy);
         return;
      end;
		self.y = self.y - dy;
   end;

   
    SetLuaGameObjectXandY( self.cppPointer, self.x, self.y );

end;


-----------------------------------------------------------------------------------------------------

function Bird:HandleRetreating()
   self.retreat_timer = self.retreat_timer - 1;
   if( self.retreat_timer <= 0 ) then
      self.retreat_timer = GenerateRandomNumber(30, self.RETREAT_TIME);
      self:TrackPlayer();
      return;
   end;

   local dx = 0;
   local dy = 0;

   if( self.x < self.playerX) then
      dx = - self.BIRD_SPEED;
   else
      dx = self.BIRD_SPEED;
   end;


	self.x = self.x + dx;

	if ( WasCollisionWithLeftRightWall(self) ) then
      self.x = self.x -dx;
	end;

--	if ( WasCollisionWithUpDownWall(self) ) then
--		self.y = self.y - dy;
--   end;


   
    SetLuaGameObjectXandY( self.cppPointer, self.x, self.y );
end;
   
-----------------------------------------------------------------------------------------------------
   
function Bird:HandleTracingWall()
--   LogLevel1("Entering Bird:HandleTracingWall()");
   self.wall_trace_timer = self.wall_trace_timer - 1;
   if(    self.wall_trace_timer  <= 0 ) then
      self.wall_trace_timer = self.WALL_TRACE_TIME;
      self:TrackPlayer();
   end;

   local dx = 0;
   local dy = 0;

   if( self.trace_direction == EAST) then
      dx = .25;
   end;

   if( self.trace_direction == WEST) then
      dx = -.25;
   end;

   if( self.trace_direction == SOUTH) then
      dy = .25;
   end;


   if( self.trace_direction == NORTH) then
      dy = -.25;
   end;


	self.x = self.x + dx;
   self.y = self.y + dy;

	if ( WasCollisionWithLeftRightWall(self) ) then
      self.x = self.x -dx;
	end;

	if ( WasCollisionWithUpDownWall(self) ) then
		self.y = self.y - dy;
   end;

   SetLuaGameObjectXandY( self.cppPointer, self.x, self.y );
--   LogLevel1("Leaving Bird:HandleTracingWall()");
end;

-----------------------------------------------------------------------------------------------------

function Bird:Update(  )


	self.playerX, self.playerY = GetPlayerXAndY();
	self.collisionBoxX, self.collisionBoxY  = GetLuaGameObjectCollisionBoxXAndY(self.cppPointer );		
--	PrintStringList( "self.playerX = " .. self.playerX, "self.playerY = " .. self.playerY );      
	self.x, self.y  = GetLuaGameObjectXAndY(self.cppPointer);      
--	PrintStringList( "self.x = " .. self.x, "self.y = " .. self.y );            
    
	self.shot_timer = self.shot_timer - 1;

	if(self.shot_timer <= 0) then
		self.shot_timer = 75;
		PrintStringList( "self.x = " .. self.x, "self.y = " .. self.y );            		
		self:CreateShot();
	end;
    
      
	
	if( self.current_state == self.NULL_STATE)  then
		self:TrackPlayer();
	elseif ( self.current_state == self.PURSUING ) then
      self:HandlePursuing();
	elseif ( self.current_state == self.RETREATING ) then
      self:HandleRetreating();
	elseif ( self.current_state == self.TRACING_WALL ) then
      self:HandleTracingWall();
   end;





--	self.PURSUING = 1;
   --self.RETREATING = 2;
   --self.TRACING_WALL = 3;
	--self.NULL_STATE = -1;
    
   

		
	AnimateLuaGameObject( self.cppPointer );
	DrawLuaGameObject( self.cppPointer );

end

-----------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------
function Bird_Create(aCppPointer)
   temp = Bird:new(aCppPointer);
   bird_objects[aCppPointer] = temp;
end;
---------------------------------------------------------------------------

function Bird_Destroy(aCppPointer)
   bird_objects[aCppPointer] = nil;
end;
---------------------------------------------------------------------------


function Bird_InitializeGameObject(aCppPointer)
   theBird = bird_objects[aCppPointer];
   theBird:InitializeGameObject();
end;



---------------------------------------------------------------------------
function Bird_Update(aCppPointer )
--   PrintStringList("Calling Bird_Update");
   theBird = bird_objects[aCppPointer];
   theBird:Update(countdownTimer);
end;


