

Grunt = {};
mt_grunt = {};
mt_grunt.__index = Grunt;
grunt_objects = {};

---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

---------------------------------------------------------------------------
function Grunt:new(aCppPointer)
   return setmetatable({cppPointer = aCppPointer},mt_grunt);
end;
---------------------------------------------------------------------------


function Grunt:RandomizeLuaGameObjectLocation( )
	
   
	local additionalOffset = 10;
	
	local gameObjectWidth = self.collisionBoxWidth + additionalOffset;
	local gameObjectHeight = self.collisionBoxHeight + additionalOffset;
		
	local minPlacementX = self.playfieldMinX + additionalOffset;
	local maxPlacementX = self.playfieldMaxX - gameObjectWidth;
	local minPlacementY = self.playfieldMinY + additionalOffset;
	local maxPlacementY = self.playfieldMaxY - gameObjectHeight;

	local playerX, playerY = GetPlayerXAndY();

   local x = 0;
   local y = 0;
   local distance = 0;

   repeat
      x = GenerateRandomNumber( minPlacementX, maxPlacementX );
      y = GenerateRandomNumber( minPlacementY, maxPlacementY );

       distance = GetDistanceBetweenTwoPoints( x,y, playerX, playerY);
   until distance >= 100;
	
	SetLuaGameObjectXandY( self.cppPointer, x, y );		
end;






function Grunt:InitializeGameObject()
--   PrintStringList("Calling Grunt:InitializeGameObject");
	self.scale = 1;
	SetLuaGameObjectScale( self.cppPointer, self.scale );
	self.bitmapWidth, self.bitmapHeight = GetLuaGameObjectBitmapSize( self.cppPointer );
        self.collisionBoxWidth = self.bitmapWidth * self.scale - 2;
	self.collisionBoxHeight = self.bitmapHeight * self.scale - 2;
	SetLuaGameObjectCollisionBoxInfo( self.cppPointer, self.collisionBoxWidth, self.collisionBoxHeight, 0 );
	self.playfieldMinX, self.playfieldMaxX, self.playfieldMinY, self.playfieldMaxY = GetPlayfieldLimits();

	self.step_distance = 1.0;
	self.age = 0;
	self.SPEEDUP_AGE_INTERVAL = 35;
	self.next_speedup_age = self.SPEEDUP_AGE_INTERVAL;



end;

---------------------------------------------------------------------------

function Grunt:PrepareForWaveStart ()
--   PrintStringList("Calling Grunt:PrepareForWaveStart");
   self.step_distance = self.step_distance / 2.0;
	self.age = 0;
	self.next_speedup_age = self.SPEEDUP_AGE_INTERVAL;
   self.explosionSpacing = 0;

   self.beamInSpacing = 14;
   self.beamInSpacingDecrementTimerMAX = 2;
   self.beamInSpacingDecrementTimer = self.beamInSpacingDecrementTimerMAX;
   self.startBeamTimerValue = GenerateRandomNumber(20,70);
   self.widthOrHeightFlag = GenerateRandomNumber(0,1);
   RandomizeLuaGameObjectLocation(self);
end

---------------------------------------------------------------------------

function Grunt:PreWaveStartUpdate(  countdownTimer)
	if countdownTimer <= self.startBeamTimerValue then
      self.beamInSpacingDecrementTimer  = self.beamInSpacingDecrementTimer -1 ;
      if( self.beamInSpacingDecrementTimer <= 0) then
         self.beamInSpacingDecrementTimer = self.beamInSpacingDecrementTimerMAX;
         self.beamInSpacing = self.beamInSpacing - 2;
         if( self.beamInSpacing < 0 ) then
            self.beamInSpacing = 0;
         end;
      end;
      ExplodingDrawLuaGameObject( self.cppPointer, self.beamInSpacing, self.widthOrHeightFlag  );
	end;
end;


function Grunt:Update( )
--   PrintStringList("Calling Grunt:Update");
	self.gruntX, self.gruntY  = GetLuaGameObjectXAndY(self.cppPointer );
	local playerX, playerY = GetPlayerXAndY();
		
   local newGruntX = self.gruntX;
	local newGruntY = self.gruntY;

	if self.gruntX > playerX then
		newGruntX = self.gruntX - self.step_distance;
	end;
		
	if self.gruntX < playerX then
		newGruntX = self.gruntX + self.step_distance
	end;
		
	if self.gruntY > playerY then
		newGruntY = self.gruntY - self.step_distance;
	end;
		
	if self.gruntY < playerY then
		newGruntY = self.gruntY + self.step_distance;
	end;

	SetLuaGameObjectXandY( self.cppPointer, newGruntX, newGruntY );

	self.age = self.age + 1;
	if( self.age >= self.next_speedup_age) then
		self.step_distance = self.step_distance + .10;
		self.next_speedup_age = self.next_speedup_age + self.SPEEDUP_AGE_INTERVAL;
	end;
		
	AnimateLuaGameObject( self.cppPointer );
	DrawLuaGameObject( self.cppPointer );

end



---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

function Grunt_Create(aCppPointer)
   temp = Grunt:new(aCppPointer);
   grunt_objects[aCppPointer] = temp;
end;

---------------------------------------------------------------------------

function Grunt_Destroy(aCppPointer)
   grunt_objects[aCppPointer] = nil;
end;

---------------------------------------------------------------------------

function Grunt_InitializeGameObject(aCppPointer)
   theGrunt = grunt_objects[aCppPointer];
   theGrunt:InitializeGameObject();
end;

---------------------------------------------------------------------------

function Grunt_PrepareForWaveStart(aCppPointer)
   theGrunt = grunt_objects[aCppPointer];
   theGrunt:PrepareForWaveStart();
end;

---------------------------------------------------------------------------

function Grunt_PreWaveStartUpdate(aCppPointer, countdownTimer)
--   PrintStringList("Calling Grunt_PreWaveStartUpdate");
   theGrunt = grunt_objects[aCppPointer];
   theGrunt:PreWaveStartUpdate(countdownTimer);
end;

---------------------------------------------------------------------------
function Grunt_Update(aCppPointer )
--   PrintStringList("Calling Grunt_Update");
   theGrunt = grunt_objects[aCppPointer];
   theGrunt:Update(countdownTimer);
end;




