

Hulk = {};
mt_hulk = {};
mt_hulk.__index = Hulk;
hulk_objects = {};

---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

---------------------------------------------------------------------------
function Hulk:new(aCppPointer)
   return setmetatable({cppPointer = aCppPointer},mt_hulk);
end;
---------------------------------------------------------------------------


function Hulk:RandomizeLuaGameObjectLocation( )
	
   
	local additionalOffset = 10;
	
	local gameObjectWidth = self.collisionBoxWidth + additionalOffset;
	local gameObjectHeight = self.collisionBoxHeight + additionalOffset;
		
	local minPlacementX = self.playfieldMinX + additionalOffset;
	local maxPlacementX = self.playfieldMaxX - gameObjectWidth;
	local minPlacementY = self.playfieldMinY + additionalOffset;
	local maxPlacementY = self.playfieldMaxY - gameObjectHeight;

	local playerX, playerY = GetPlayerXAndY();

   local x = 0;
   local y = 0;
   local distance = 0;

   repeat
--   PrintStringList(minPlacementX, maxPlacementX,minPlacementY , maxPlacementY);
      x = GenerateRandomNumber( minPlacementX, maxPlacementX );
      y = GenerateRandomNumber( minPlacementY, maxPlacementY );

       distance = GetDistanceBetweenTwoPoints( x,y, playerX, playerY);
   until distance >= 100;
	
--      PrintStringList("x = " .. x, "y = " .. y );
	SetLuaGameObjectXandY( self.cppPointer, x, y );		
end;


---------------------------------------------------------------------------

function Hulk:IsOppositeDirection( direction1, direction2)
	
   local isOpposite = false;


   if( direction1 == self.HULK_MOTION_DIRECTION_RIGHT) then
		if( direction2 == self.HULK_MOTION_DIRECTION_LEFT) then
         isOpposite = true;
		end;
	elseif ( direction1 == self.HULK_MOTION_DIRECTION_LEFT ) then
		if( direction2 == self.HULK_MOTION_DIRECTION_RIGHT) then
			isOpposite = true;
       end;
   elseif ( direction1 == self.HULK_MOTION_DIRECTION_UP ) then
		if( direction2 == self.HULK_MOTION_DIRECTION_DOWN) then
			isOpposite = true;
      end;
   elseif ( direction1 == self.HULK_MOTION_DIRECTION_DOWN ) then
		if( direction2 == self.HULK_MOTION_DIRECTION_UP) then 
			isOpposite = true;
      end;
   end;

	return isOpposite;

end;

---------------------------------------------------------------------------

function Hulk:SelectNewDirection()
--   PrintStringList("Calling Hulk:SelectNewDirection");
	local oldDirection = self.motionDirection;

	self.motionDirection = GenerateRandomNumber(0,3);

	while( self:IsOppositeDirection( oldDirection, self.motionDirection) )
   do
      self.motionDirection = GenerateRandomNumber(0,3);
   end;

	self.motionCountdownTimer = GenerateRandomNumber(25, 60 );
--   PrintStringList("Exiting Hulk:SelectNewDirection");
end;



---------------------------------------------------------------------------


function Hulk:InitializeGameObject()
--   PrintStringList("Calling Hulk:InitializeGameObject");
	self.scale = 1;
	SetLuaGameObjectScale( self.cppPointer, self.scale );
	self.bitmapWidth, self.bitmapHeight = GetLuaGameObjectBitmapSize( self.cppPointer );
   self.collisionBoxWidth = self.bitmapWidth * self.scale;
	self.collisionBoxHeight = self.bitmapHeight * self.scale;
	SetLuaGameObjectCollisionBoxInfo( self.cppPointer, self.collisionBoxWidth, self.collisionBoxHeight, 0 );
	self.playfieldMinX, self.playfieldMaxX, self.playfieldMinY, self.playfieldMaxY = GetPlayfieldLimits();



   self. HULK_MOTION_DIRECTION_RIGHT = 0;
   self. HULK_MOTION_DIRECTION_DOWN = 1;
   self. HULK_MOTION_DIRECTION_LEFT = 2;
   self. HULK_MOTION_DIRECTION_UP = 3;
   self. MAX_HULK_MOTION_DIRECTION_DISTANCE = 10;
   self.HULK_SPEED  = 1;

	self.motionDirection = MOTION_DIRECTION_RIGHT;
	self:SelectNewDirection();

   SetAnimationSpeed(self.cppPointer, 8);


end;

---------------------------------------------------------------------------

function Hulk:PrepareForWaveStart ()

   self.beamInSpacing = 14;
   self.beamInSpacingDecrementTimerMAX = 2;
   self.beamInSpacingDecrementTimer = self.beamInSpacingDecrementTimerMAX;
   self.startBeamTimerValue = GenerateRandomNumber(20,70);
   self.widthOrHeightFlag = GenerateRandomNumber(0,1);



   self:RandomizeLuaGameObjectLocation();
end

---------------------------------------------------------------------------

function Hulk:PreWaveStartUpdate(  countdownTimer)
	if countdownTimer <= self.startBeamTimerValue then
      self.beamInSpacingDecrementTimer  = self.beamInSpacingDecrementTimer -1 ;
      if( self.beamInSpacingDecrementTimer <= 0) then
         self.beamInSpacingDecrementTimer = self.beamInSpacingDecrementTimerMAX;
         self.beamInSpacing = self.beamInSpacing - 2;
         if( self.beamInSpacing < 0 ) then
            self.beamInSpacing = 0;
         end;
      end;
      ExplodingDrawLuaGameObject( self.cppPointer, self.beamInSpacing, self.widthOrHeightFlag  );
	end;

end;


function Hulk:Update( )
--   PrintStringList("Calling Hulk:Update");
	self.x, self.y  = GetLuaGameObjectXAndY(self.cppPointer );

	self.motionCountdownTimer = self.motionCountdownTimer  - 1;
	if(self.motionCountdownTimer <= 0) then
		self:SelectNewDirection();
   end;

	local dx = 0;
	local dy = 0;

   local currentAnimation = GetAnimation(self.cppPointer) ;

	if( self.motionDirection ==  self.HULK_MOTION_DIRECTION_RIGHT) then
--			self.x +=self.HULK_SPEED;
		dx = self.HULK_SPEED;
		if (currentAnimation ~= EAST) then
			SetAnimation(self.cppPointer,EAST);
        end;
   end;

	if( self.motionDirection ==  self.HULK_MOTION_DIRECTION_LEFT) then
--			self.x -= HULK_SPEED;
		dx = -self.HULK_SPEED;
      if (currentAnimation ~= WEST) then
			SetAnimation(self.cppPointer,WEST);
      end;
   end;

	if( self.motionDirection ==  self.HULK_MOTION_DIRECTION_UP) then
--			self.y -= HULK_SPEED;
		dy = - self.HULK_SPEED;
		if (currentAnimation ~= NORTH) then
			SetAnimation(self.cppPointer,NORTH);
        end;
   end;

	if( self.motionDirection ==  self.HULK_MOTION_DIRECTION_DOWN) then
--			self.y += HULK_SPEED;
		dy = self.HULK_SPEED;
		if (currentAnimation ~= SOUTH) then
			SetAnimation(self.cppPointer,SOUTH);
		end;
   end;



   self.x = self.x + dx;
   self.y = self.y + dy;

	if ( WasCollisionWithLeftRightWall(self) ) then
      self.x = self.x -dx;
		self:SelectNewDirection();
	end;

	if ( WasCollisionWithUpDownWall(self) ) then
		self.y = self.y - dy;
		self:SelectNewDirection();
   end;

	SetLuaGameObjectXandY( self.cppPointer, self.x,self.y);

	AnimateLuaGameObject( self.cppPointer );
	DrawLuaGameObject( self.cppPointer );


end



---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

function Hulk_GetLuaObjectFromCppPointer( aCppPointer )
   return hulk_objects[aCppPointer];
end;


function Hulk_Create(aCppPointer)
   temp = Hulk:new(aCppPointer);
   hulk_objects[aCppPointer] = temp;
end;

---------------------------------------------------------------------------

function Hulk_Destroy(aCppPointer)
   hulk_objects[aCppPointer] = nil;
end;

---------------------------------------------------------------------------

function Hulk_InitializeGameObject(aCppPointer)
   theHulk = hulk_objects[aCppPointer];
   theHulk:InitializeGameObject();
end;

---------------------------------------------------------------------------

function Hulk_PrepareForWaveStart(aCppPointer)
   theHulk = hulk_objects[aCppPointer];
   theHulk:PrepareForWaveStart();
end;

---------------------------------------------------------------------------

function Hulk_PreWaveStartUpdate(aCppPointer, countdownTimer)
--   PrintStringList("Calling Hulk_PreWaveStartUpdate");
   theHulk = hulk_objects[aCppPointer];
   theHulk:PreWaveStartUpdate(countdownTimer);
end;

---------------------------------------------------------------------------
function Hulk_Update(aCppPointer )
--   PrintStringList("Calling Hulk_Update");
   theHulk = hulk_objects[aCppPointer];
   theHulk:Update(countdownTimer);
end;




