
Tank = {};
mt_tank = {};
mt_tank.__index = Tank;
tank_objects = {};


---------------------------------------------------------------------------
function Tank:new(aCppPointer)
   return setmetatable({cppPointer = aCppPointer},mt_tank);
end;
---------------------------------------------------------------------------

function Tank:RandomizeLuaGameObjectLocation( )
	
   
	local additionalOffset = 10;
	
	local gameObjectWidth = self.collisionBoxWidth + additionalOffset;
	local gameObjectHeight = self.collisionBoxHeight + additionalOffset;
		
	local minPlacementX = self.playfieldMinX + additionalOffset;
	local maxPlacementX = self.playfieldMaxX - gameObjectWidth;
	local minPlacementY = self.playfieldMinY + additionalOffset;
	local maxPlacementY = self.playfieldMaxY - gameObjectHeight;

	local playerX, playerY = GetPlayerXAndY();

   local x = 0;
   local y = 0;
   local distance = 0;

   repeat
--   PrintStringList(minPlacementX, maxPlacementX,minPlacementY , maxPlacementY);
      x = GenerateRandomNumber( minPlacementX, maxPlacementX );
      y = GenerateRandomNumber( minPlacementY, maxPlacementY );

       distance = GetDistanceBetweenTwoPoints( x,y, playerX, playerY);
   until distance >= 100;
	
--      PrintStringList("x = " .. x, "y = " .. y );
	SetLuaGameObjectXandY( self.cppPointer, x, y );		
end;



---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------


	
-----------------------------------------------------------------------------------------------------
------------------------ Required functions ---------------------------------------------------------
-----------------------------------------------------------------------------------------------------
function Tank:InitializeGameObject(  )
	self.scale = 1.3;
	SetLuaGameObjectScale( self.cppPointer, self.scale );
	self.bitmapWidth, self.bitmapHeight = GetLuaGameObjectBitmapSize( self.cppPointer );
	self.collisionBoxWidth = self.bitmapWidth * self.scale;
	self.collisionBoxHeight = self.bitmapHeight * self.scale;
	SetLuaGameObjectCollisionBoxInfo( self.cppPointer, self.collisionBoxWidth, self.collisionBoxHeight, 0 );
	self.playfieldMinX, self.playfieldMaxX, self.playfieldMinY, self.playfieldMaxY = GetPlayfieldLimits();




	self.shot_timer = 75;
	

   self.motionCountdownTimer = 0;

  
   self.TANK_SPEED = 1;
   self.MOTION_DIRECTION_RIGHT = 0;
   self.MOTION_DIRECTION_DOWN = 1;
   self.MOTION_DIRECTION_LEFT = 2;
   self.MOTION_DIRECTION_UP = 3;

   self.MOTION_DIRECTION_RIGHT_UP = 4;
   self.MOTION_DIRECTION_LEFT_UP = 5;
   self.MOTION_DIRECTION_RIGHT_DOWN = 6;
   self.MOTION_DIRECTION_LEFT_DOWN = 7;

   self.motionDirection = 0;

   self.MAX_MOTION_DIRECTION_DISTANCE = 60;


end

-----------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------

function Tank:PrepareForWaveStart ()
   self.beamInSpacing = 14;
   self.beamInSpacingDecrementTimerMAX = 2;
   self.beamInSpacingDecrementTimer = self.beamInSpacingDecrementTimerMAX;
   self.startBeamTimerValue = GenerateRandomNumber(20,70);
   self.widthOrHeightFlag = GenerateRandomNumber(0,1);

   self:RandomizeLuaGameObjectLocation();
end

---------------------------------------------------------------------------

function Tank:PreWaveStartUpdate(  countdownTimer)

	if countdownTimer <= self.startBeamTimerValue then
      self.beamInSpacingDecrementTimer  = self.beamInSpacingDecrementTimer -1 ;
      if( self.beamInSpacingDecrementTimer <= 0) then
         self.beamInSpacingDecrementTimer = self.beamInSpacingDecrementTimerMAX;
         self.beamInSpacing = self.beamInSpacing - 2;
         if( self.beamInSpacing < 0 ) then
            self.beamInSpacing = 0;
         end;
      end;
      ExplodingDrawLuaGameObject( self.cppPointer, self.beamInSpacing, self.widthOrHeightFlag  );
	end;
end;

-----------------------------------------------------------------------------------------------------
   

   
function Tank:CreateShot()

	self.playerX, self.playerY = GetPlayerXAndY();


	local xv = GenerateRandomNumber(2,5);
	local yv = GenerateRandomNumber(2,5);

	if( self.playerX <= self.x ) then
		xv = -xv;
	end;

	if( self.playerY <= self.y ) then
		yv = -yv;
	end;


	local addedTankShot = AddGameObject(TANK_SHOT_TYPE, self.x + 10, self.y +10 );
	SetLuaGameObjectXVAndYV( addedTankShot, xv,yv );


end;
-----------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------------------
function Tank:SelectNewDirection()


	local oldDirection = self.motionDirection;

	self.motionDirection = GenerateRandomNumber(0,7);

--	while( IsOppositeDirection( oldDirection, self.motionDirection) )
--	{
		self.motionDirection = GenerateRandomNumber(0,7);
--	}
	self.motionCountdownTimer = GenerateRandomNumber(20, self.MAX_MOTION_DIRECTION_DISTANCE);

end;


-----------------------------------------------------------------------------------------------------

function WasCollisionWithLeftRightWall(self)
	return ( WasCollisionWithLeftWall(self) or WasCollisionWithRightWall(self) );
end;
-----------------------------------------------------------------------------------------------------
function WasCollisionWithUpDownWall(self)
	return  (	WasCollisionWithUpperWall(self) or WasCollisionWithLowerWall(self) );
end;

-----------------------------------------------------------------------------------------------------
function WasCollisionWithLeftWall(self)

	return (self.x < (self.playfieldMinX + 10 ));
--	return (self.collisionBoxX < (self.playfieldMinX + 5 ));	
end;

-----------------------------------------------------------------------------------------------------

function WasCollisionWithRightWall(self)
	return 	(self.x + self.collisionBoxWidth )  > (self.playfieldMaxX - 10);
--	return 	(self.collisionBoxX + self.collisionBoxWidth )  > self.playfieldMaxX;	
end;

-----------------------------------------------------------------------------------------------------

function WasCollisionWithUpperWall(self)
	return (self.y < (self.playfieldMinY + 5));
--	return (self.collisionBoxY < (self.playfieldMinY + 5));	
end;


-----------------------------------------------------------------------------------------------------

function WasCollisionWithLowerWall(self)
   local result = false;

   if ( ( self.y + 	self.collisionBoxHeight ) > (self.playfieldMaxY - 5) ) then
      result = true;
   end;

--   if ( ( self.collisionBoxY + 	self.collisionBoxHeight ) > self.playfieldMaxY ) then
--      result = true;
--   end;


   return result;
end;



   
-----------------------------------------------------------------------------------------------------

function Tank:Update(  )

	self.collisionBoxX, self.collisionBoxY  = GetLuaGameObjectCollisionBoxXAndY(self.cppPointer );	

	self.shot_timer = self.shot_timer - 1;

	if(self.shot_timer <= 0) then
		self.shot_timer = GenerateRandomNumber(100,200);
		self:CreateShot(GenerateRandomNumber(4,6),GenerateRandomNumber(4,6));
   end;


	self.motionCountdownTimer = self.motionCountdownTimer - 1;
	if(self.motionCountdownTimer <= 0) then
			self:SelectNewDirection();
	end;


	local dx = 0;
	local dy = 0;

	if( self.motionDirection == self.MOTION_DIRECTION_RIGHT ) then
		dx = self.TANK_SPEED;
	elseif ( self.motionDirection == self.MOTION_DIRECTION_LEFT ) then
		dx = -self.TANK_SPEED;
	elseif ( self.motionDirection ==  self.MOTION_DIRECTION_UP ) then
			dy = -self.TANK_SPEED;
   elseif ( self.motionDirection ==  self.MOTION_DIRECTION_DOWN ) then
			dy = self.TANK_SPEED;
   elseif ( self.motionDirection ==  self.MOTION_DIRECTION_RIGHT_UP ) then
			dx = self.TANK_SPEED;
			dy = -self.TANK_SPEED;
   elseif ( self.motionDirection ==  self.MOTION_DIRECTION_LEFT_UP ) then
			dx = -self.TANK_SPEED;
			dy = -self.TANK_SPEED;
   elseif ( self.motionDirection ==  self.MOTION_DIRECTION_RIGHT_DOWN ) then
			dx = self.TANK_SPEED;
			dy = self.TANK_SPEED;
   elseif ( self.motionDirection ==  self.MOTION_DIRECTION_LEFT_DOWN ) then
			dx = -self.TANK_SPEED;
			dy = self.TANK_SPEED;
   end;

	self.x, self.y  = GetLuaGameObjectXAndY(self.cppPointer);      

	self.x = self.x + dx;
	self.y = self.y + dy;


	if ( WasCollisionWithLeftRightWall(self) ) then
      self.x = self.x - dx;
		self:SelectNewDirection();
   end;

	if ( WasCollisionWithUpDownWall(self) ) then
      self.y = self.y - dy;
		self:SelectNewDirection();
   end;



   SetLuaGameObjectXandY( self.cppPointer, self.x, self.y );

	
	AnimateLuaGameObject( self.cppPointer );
	DrawLuaGameObject( self.cppPointer );

end

-----------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

function Tank_Create(aCppPointer)
   temp = Tank:new(aCppPointer);
   tank_objects[aCppPointer] = temp;
end;

---------------------------------------------------------------------------

function Tank_Destroy(aCppPointer)
   tank_objects[aCppPointer] = nil;
end;

---------------------------------------------------------------------------



function Tank_InitializeGameObject(aCppPointer)
   theTank = tank_objects[aCppPointer];
   theTank:InitializeGameObject();
end;

---------------------------------------------------------------------------

function Tank_PrepareForWaveStart(aCppPointer)
   theTank = tank_objects[aCppPointer];
   theTank:PrepareForWaveStart();
end;

---------------------------------------------------------------------------

function Tank_PreWaveStartUpdate(aCppPointer, countdownTimer)
--   PrintStringList("Calling Tank_PreWaveStartUpdate");
   theTank = tank_objects[aCppPointer];
   theTank:PreWaveStartUpdate(countdownTimer);
end;



---------------------------------------------------------------------------
function Tank_Update(aCppPointer )
--   PrintStringList("Calling Tank_Update");
   theTank = tank_objects[aCppPointer];
   theTank:Update(countdownTimer);
end;


