
Triangle = {};
mt_triangle = {};
mt_triangle.__index = Triangle;
triangle_objects = {};

---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

function Triangle:new(aCppPointer)
   return setmetatable({cppPointer = aCppPointer},mt_triangle);
end;
---------------------------------------------------------------------------


---- "Instance" variables ----------------------------
-- Each Triangle will have one instance of each of these variables
-- so you can think of this file as representing the Triangle as a class with the instance variables
-- and methods shown below

-- The Triangle's behavior is pretty simple. It has two states:
--		* MOVING_TOWARDS_CORNER
--		* PAUSING_IN_CORNER.
-- It goes back and forth between these two states.
-- It also decrements bird_emission_timer every cycle and starts
-- emitting birds when it reaches zero.
-- After all the birds are emitted, it will then stick around
-- a few more cycles and then remove itself from the wave


-- The function "InitializeBeforeWaveStart" is called before each wave is started
-- or re-started
-- The function "Update" is called every game cycle
-- Any functions called that are defined in this Triangle.lua file are C++ callback functions

   
---- Functions --------------------------------------------------------------------------------

function Triangle:SelectNewCorner()
		local oldCorner = self.corner_moving_towards;
		
		self.corner_moving_towards = GenerateRandomNumber(0,3);		
		
		while self.corner_moving_towards == oldCorner do
			self.corner_moving_towards = GenerateRandomNumber(0,3);
		end;
		

		self.corner_wait_timer = GenerateRandomNumber(20,60);
		self.current_state = self.MOVING_TOWARDS_CORNER;

		
end;
-----------------------------------------------------------------------------------------------------

function Triangle:HandleNullState()
	self:SelectNewCorner();
end;

-----------------------------------------------------------------------------------------------------
function Triangle:HandleMovingTowardsNorthWestCorner()
	local triangleX, triangleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local newX = triangleX;
	local newY = triangleY;

    local movedX =0;
    local movedY = 0;
		
		if triangleX >self.playfieldMinX then
			newX = newX - self.CIRCLE_SPEED;
         movedX = 1;
		end;
		
		if (triangleY - self.CIRCLE_SPEED) > self.playfieldMinY then
			newY = newY - self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			

		return newX, newY;
	
end;

-----------------------------------------------------------------------------------------------------
function Triangle:HandleMovingTowardsNorthEastCorner()
	local	triangleX, triangleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local		newX = triangleX;
	local		newY = triangleY;

	local      movedX =0;
	local      movedY = 0;


      if ((triangleX + self.CIRCLE_SPEED + self.collisionBoxWidth ) < self.playfieldMaxX) then
			newX = newX + self.CIRCLE_SPEED;
         movedX = 1;
		end;
		
		if triangleY - self.CIRCLE_SPEED  > self.playfieldMinY then
			newY = newY - self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			


		return newX, newY;
	
end;

-----------------------------------------------------------------------------------------------------
function Triangle:HandleMovingTowardsSouthEastCorner()

	local triangleX, triangleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local newX = triangleX;
	local newY = triangleY;

    local movedX =0;
    local movedY = 0;

		
		if (triangleX + self.CIRCLE_SPEED + self.collisionBoxWidth)  < self.playfieldMaxX then
			newX = newX + self.CIRCLE_SPEED;
         movedX = 1;
		end;
		

		if (triangleY +self.CIRCLE_SPEED +self.collisionBoxHeight ) < self.playfieldMaxY then
			newY = newY + self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			

		return newX, newY;
	
end;

-----------------------------------------------------------------------------------------------------
function Triangle:HandleMovingTowardsSouthWestCorner()
	local	triangleX, triangleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local		newX = triangleX;
	local	newY = triangleY;

	local      movedX =0;
	local      movedY = 0;


		if triangleX  > self.playfieldMinX then
			newX = newX - self.CIRCLE_SPEED;
         movedX = 1;
		end;
		

		if (triangleY +self.CIRCLE_SPEED + self.collisionBoxHeight ) < self.playfieldMaxY then
			newY = newY + self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			

		return newX, newY;
	
end;


-----------------------------------------------------------------------------------------------------
function Triangle:HandleMovingTowardsCorner()
	
	local newX = 0;
	local newY = 0;
	
		if self.corner_moving_towards == self.NORTH_WEST_CORNER then
			newX, newY = self:HandleMovingTowardsNorthWestCorner(self.cppPointer);
		elseif self.corner_moving_towards == self.NORTH_EAST_CORNER then
			newX, newY = self:HandleMovingTowardsNorthEastCorner(self.cppPointer);
		elseif self.corner_moving_towards == self.SOUTH_EAST_CORNER then
			newX, newY = self:HandleMovingTowardsSouthEastCorner(self.cppPointer);
		elseif self.corner_moving_towards == self.SOUTH_WEST_CORNER then
			newX, newY = self:HandleMovingTowardsSouthWestCorner(self.cppPointer);
		end;
		
		SetLuaGameObjectXandY( self.cppPointer, newX, newY );		
	
end;
	
-----------------------------------------------------------------------------------------------------	
function Triangle:HandlePausingInCorner()
		self.corner_wait_timer = self.corner_wait_timer - 1;
		if self.corner_wait_timer <= 0 then
			self:SelectNewCorner();
		end;
	
end;


-----------------------------------------------------------------------------------------------------
function Triangle:GetMaxXForPlacement()

      return self.playfieldMaxX - self.collisionBoxWidth - 5;

end;
-----------------------------------------------------------------------------------------------------
function Triangle:GetMinXForPlacement()

      return self.playfieldMinX + 5;
end;
-----------------------------------------------------------------------------------------------------
function Triangle:GetMaxYForPlacement()

      return self.playfieldMaxY - self.collisionBoxHeight - 5;
end;
-----------------------------------------------------------------------------------------------------
function Triangle:GetMinYForPlacement()
   
      return self.playfieldMinY + 5;
end;

-----------------------------------------------------------------------------------------------------
function Triangle:AddNewTank()

	local triangleX, triangleY  = GetLuaGameObjectXAndY(self.cppPointer);

   local newTankCppPointer = AddGameObject( TANK_TYPE, triangleX, triangleY );	
   local dummyTank = {};

   

   dummyTank.x, dummyTank.y = GetLuaGameObjectXAndY(newTankCppPointer);
   dummyTank.collisionBoxWidth, dummyTank.collisionBoxHeight = GetLuaGameObjectCollisionBoxInfo( newTankCppPointer );
	dummyTank.playfieldMinX, dummyTank.playfieldMaxX, dummyTank.playfieldMinY, dummyTank.playfieldMaxY = GetPlayfieldLimits();

	while(WasCollisionWithUpperWall(dummyTank) 	) do
       dummyTank.y = dummyTank.y + 1;
   end;

	while(WasCollisionWithLowerWall(dummyTank) 	) do
       dummyTank.y = dummyTank.y - 1;   
   end;

	while(WasCollisionWithLeftWall(dummyTank) 	) do
       dummyTank.x = dummyTank.x + 1;   
   end;

	while(WasCollisionWithRightWall(dummyTank) 	) do
       dummyTank.x = dummyTank.x - 1;   
   end;


   SetLuaGameObjectXandY( newTankCppPointer, dummyTank.x, dummyTank.y );
   

end;


-----------------------------------------------------------------------------------------------------
------------------------ Required functions ---------------------------------------------------------
-----------------------------------------------------------------------------------------------------
function Triangle:InitializeGameObject( )
	self.scale = 1;
	SetLuaGameObjectScale( self.cppPointer, self.scale );
	self.bitmapWidth, self.bitmapHeight = GetLuaGameObjectBitmapSize( self.cppPointer );
    self.collisionBoxWidth = self.bitmapWidth * self.scale;
	self.collisionBoxHeight = self.bitmapHeight * self.scale;
	SetLuaGameObjectCollisionBoxInfo( self.cppPointer, self.collisionBoxWidth, self.collisionBoxHeight, 0 );
	self.playfieldMinX, self.playfieldMaxX, self.playfieldMinY, self.playfieldMaxY = GetPlayfieldLimits();

	self.CIRCLE_SPEED = 4;
	
--  Triangle state constants
	self.MOVING_TOWARDS_CORNER = 0;
	self.PAUSING_IN_CORNER = 1;
	self.NULL_STATE = -1;


--  Corner name constants
	self.NORTH_WEST_CORNER = 0;
	self.NORTH_EAST_CORNER = 1;
	self.SOUTH_EAST_CORNER = 2;
	self.SOUTH_WEST_CORNER = 3;

	self.CORNER_WAIT_TIME = 60;

	self.current_state = NULL_STATE;
	self.corner_moving_towards = 1;
	
	self.corner_wait_timer = CORNER_WAIT_TIME;
	
	self.remaining_birds = 5;
	
	self.bird_emission_timer	= 100;
	
	self.beam_out_timer = 100;


end

-----------------------------------------------------------------------------------------------------

function Triangle:PrepareForWaveStart ( )
		self:SelectNewCorner();
		self.bird_emission_timer = GenerateRandomNumber(100, 200);
		self.remaining_birds = 5;

      local newX = 0;
      local newY = 0;

		local starting_side = GenerateRandomNumber(0,3);
		if		(  starting_side == 0 ) then
			-- top side
			newX = GenerateRandomNumber(self:GetMinXForPlacement(),self:GetMaxXForPlacement());
			newY = self:GetMinYForPlacement();
		elseif	(  starting_side == 1 ) then
			-- right side
			newX = self:GetMaxXForPlacement();
			newY = GenerateRandomNumber(self:GetMinYForPlacement(),self:GetMaxYForPlacement() );
			
		elseif	(  starting_side == 2 ) then
			-- bottom
			newX = GenerateRandomNumber(self:GetMinXForPlacement(),self:GetMaxXForPlacement());
			newY = self:GetMaxYForPlacement();
		
		
		elseif	(  starting_side == 3 ) then		
			-- left
			newX = self:GetMinXForPlacement();
			newY = GenerateRandomNumber(self:GetMinYForPlacement(),self:GetMaxYForPlacement() );
		end

		SetLuaGameObjectXandY( self.cppPointer, newX, newY );					

end
-----------------------------------------------------------------------------------------------------
function Triangle:PreWaveStartUpdate( countdownTimer)
		if countdownTimer <= 70 then
         AnimateLuaGameObject( self.cppPointer );
			DrawLuaGameObject( self.cppPointer );
		end;
end;

-----------------------------------------------------------------------------------------------------

function Triangle:Update ()
	
--		PrintStringList( "self.current_state = " .. self.current_state );
	
		if self.current_state == NULL_STATE then
			self:HandleNullState();
		end;
		
		if self.current_state == self.MOVING_TOWARDS_CORNER then
			self:HandleMovingTowardsCorner(self.cppPointer);
		end;
		
		if self.current_state == self.PAUSING_IN_CORNER then
			self:HandlePausingInCorner();
		end;

		if self.remaining_birds > 0 then
			self.bird_emission_timer = self.bird_emission_timer - 1;
			if self.bird_emission_timer <= 0 then
				self.bird_emission_timer = 10;
				self.remaining_birds = self.remaining_birds - 1;
--				local triangleX, triangleY  = GetLuaGameObjectXAndY(self.cppPointer);
--				AddGameObject( TANK_TYPE, triangleX, triangleY );
            self:AddNewTank();
--				PlaySound( BIRD_EMITTED_SOUND);				
			end;
		else
			self.beam_out_timer = self.beam_out_timer - 1;
			if self.beam_out_timer <= 0 then
            SetLuaGameObjectIsDeadFlag( self.cppPointer,1 );
			end;
		end;		
	
		AnimateLuaGameObject( self.cppPointer );
		DrawLuaGameObject( self.cppPointer );
	

end
-----------------------------------------------------------------------------------------------------	



---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------
function Triangle_Create(aCppPointer)
   temp = Triangle:new(aCppPointer);
   triangle_objects[aCppPointer] = temp;
end;
---------------------------------------------------------------------------
function Triangle_Destroy(aCppPointer)
   triangle_objects[aCppPointer] = nil;
end;
---------------------------------------------------------------------------


function Triangle_InitializeGameObject(aCppPointer)
   theTriangle = triangle_objects[aCppPointer];
   theTriangle:InitializeGameObject();
end;

---------------------------------------------------------------------------

function Triangle_PrepareForWaveStart(aCppPointer)
   theTriangle = triangle_objects[aCppPointer];
   theTriangle:PrepareForWaveStart();
end;

---------------------------------------------------------------------------

function Triangle_PreWaveStartUpdate(aCppPointer, countdownTimer)
--   PrintStringList("Calling Triangle_PreWaveStartUpdate");
   theTriangle = triangle_objects[aCppPointer];
   theTriangle:PreWaveStartUpdate(countdownTimer);
end;

---------------------------------------------------------------------------
function Triangle_Update(aCppPointer )
--   PrintStringList("Calling Triangle_Update");
   theTriangle = triangle_objects[aCppPointer];
   theTriangle:Update(countdownTimer);
end;


