
Circle = {};
mt_circle = {};
mt_circle.__index = Circle;
circle_objects = {};

---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

function Circle:new(aCppPointer)
   return setmetatable({cppPointer = aCppPointer},mt_circle);
end;
---------------------------------------------------------------------------


---- "Instance" variables ----------------------------
-- Each Circle will have one instance of each of these variables
-- so you can think of this file as representing the Circle as a class with the instance variables
-- and methods shown below

-- The Circle's behavior is pretty simple. It has two states:
--		* MOVING_TOWARDS_CORNER
--		* PAUSING_IN_CORNER.wo states.
-- It also decrements bird_emission_timer every cycle and starts
-- emitting birds when it reaches zero.
-- After all the birds are emitted, it will then stick around
-- a few more cycles and then remove itself from the wave


-- The function "InitializeBeforeWaveStart" is called before each wave is started
-- or re-started
-- The function "Update" is called every game cycle

   
---- Functions --------------------------------------------------------------------------------

function Circle:SelectNewCorner()
		local oldCorner = self.corner_moving_towards;
		
		self.corner_moving_towards = GenerateRandomNumber(0,3);		
		
		while self.corner_moving_towards == oldCorner do
			self.corner_moving_towards = GenerateRandomNumber(0,3);
		end;
		

		self.corner_wait_timer = GenerateRandomNumber(20,60);
		self.current_state = self.MOVING_TOWARDS_CORNER;

		
end;
-----------------------------------------------------------------------------------------------------

function Circle:HandleNullState()
	self:SelectNewCorner();
end;

-----------------------------------------------------------------------------------------------------
function Circle:HandleMovingTowardsNorthWestCorner()
	local circleX, circleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local newX = circleX;
	local newY = circleY;

    local movedX =0;
    local movedY = 0;
		
		if circleX >self.playfieldMinX then
			newX = newX - self.CIRCLE_SPEED;
         movedX = 1;
		end;
		
		if (circleY - self.CIRCLE_SPEED) > self.playfieldMinY then
			newY = newY - self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			

		return newX, newY;
	
end;

-----------------------------------------------------------------------------------------------------
function Circle:HandleMovingTowardsNorthEastCorner()
	local	circleX, circleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local		newX = circleX;
	local		newY = circleY;

	local      movedX =0;
	local      movedY = 0;


      if ((circleX + self.CIRCLE_SPEED + self.collisionBoxWidth ) < self.playfieldMaxX) then
			newX = newX + self.CIRCLE_SPEED;
         movedX = 1;
		end;
		
		if circleY - self.CIRCLE_SPEED  > self.playfieldMinY then
			newY = newY - self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			


		return newX, newY;
	
end;

-----------------------------------------------------------------------------------------------------
function Circle:HandleMovingTowardsSouthEastCorner()

	local circleX, circleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local newX = circleX;
	local newY = circleY;

    local movedX =0;
    local movedY = 0;

		
		if (circleX + self.CIRCLE_SPEED + self.collisionBoxWidth)  < self.playfieldMaxX then
			newX = newX + self.CIRCLE_SPEED;
         movedX = 1;
		end;
		

		if (circleY +self.CIRCLE_SPEED +self.collisionBoxHeight ) < self.playfieldMaxY then
			newY = newY + self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			

		return newX, newY;
	
end;

-----------------------------------------------------------------------------------------------------
function Circle:HandleMovingTowardsSouthWestCorner()
	local	circleX, circleY  = GetLuaGameObjectXAndY(self.cppPointer);
	local		newX = circleX;
	local	newY = circleY;

	local      movedX =0;
	local      movedY = 0;


		if circleX  > self.playfieldMinX then
			newX = newX - self.CIRCLE_SPEED;
         movedX = 1;
		end;
		

		if (circleY +self.CIRCLE_SPEED + self.collisionBoxHeight ) < self.playfieldMaxY then
			newY = newY + self.CIRCLE_SPEED;
         movedY = 1;
		end;

		if (movedX == 0 )  then
			if( movedY == 0)  then
				self.current_state = self.PAUSING_IN_CORNER;
			end;
		end;			

		return newX, newY;
	
end;


-----------------------------------------------------------------------------------------------------
function Circle:HandleMovingTowardsCorner()
	
	local newX = 0;
	local newY = 0;
	
		if self.corner_moving_towards == self.NORTH_WEST_CORNER then
			newX, newY = self:HandleMovingTowardsNorthWestCorner(self.cppPointer);
		elseif self.corner_moving_towards == self.NORTH_EAST_CORNER then
			newX, newY = self:HandleMovingTowardsNorthEastCorner(self.cppPointer);
		elseif self.corner_moving_towards == self.SOUTH_EAST_CORNER then
			newX, newY = self:HandleMovingTowardsSouthEastCorner(self.cppPointer);
		elseif self.corner_moving_towards == self.SOUTH_WEST_CORNER then
			newX, newY = self:HandleMovingTowardsSouthWestCorner(self.cppPointer);
		end;
		
		SetLuaGameObjectXandY( self.cppPointer, newX, newY );		
	
end;
	
-----------------------------------------------------------------------------------------------------	
function Circle:HandlePausingInCorner()
		self.corner_wait_timer = self.corner_wait_timer - 1;
		if self.corner_wait_timer <= 0 then
			self:SelectNewCorner();
		end;
	
end;


-----------------------------------------------------------------------------------------------------
function Circle:GetMaxXForPlacement()

      return self.playfieldMaxX - self.collisionBoxWidth - 5;

end;
-----------------------------------------------------------------------------------------------------
function Circle:GetMinXForPlacement()

      return self.playfieldMinX + 5;
end;
-----------------------------------------------------------------------------------------------------
function Circle:GetMaxYForPlacement()

      return self.playfieldMaxY - self.collisionBoxHeight - 5;
end;
-----------------------------------------------------------------------------------------------------
function Circle:GetMinYForPlacement()
   
      return self.playfieldMinY + 5;
end;


-----------------------------------------------------------------------------------------------------
------------------------ Required functions ---------------------------------------------------------
-----------------------------------------------------------------------------------------------------
function Circle:InitializeGameObject( )
	self.scale = 1;
	SetLuaGameObjectScale( self.cppPointer, self.scale );
	self.bitmapWidth, self.bitmapHeight = GetLuaGameObjectBitmapSize( self.cppPointer );
    self.collisionBoxWidth = self.bitmapWidth * self.scale -5;
	self.collisionBoxHeight = self.bitmapHeight * self.scale -5;
	SetLuaGameObjectCollisionBoxInfo( self.cppPointer, self.collisionBoxWidth, self.collisionBoxHeight, 0 );
	self.playfieldMinX, self.playfieldMaxX, self.playfieldMinY, self.playfieldMaxY = GetPlayfieldLimits();

	self.CIRCLE_SPEED = 4;
	
--  Circle state constants
	self.MOVING_TOWARDS_CORNER = 0;
	self.PAUSING_IN_CORNER = 1;
	self.NULL_STATE = -1;


--  Corner name constants
	self.NORTH_WEST_CORNER = 0;
	self.NORTH_EAST_CORNER = 1;
	self.SOUTH_EAST_CORNER = 2;
	self.SOUTH_WEST_CORNER = 3;

	self.CORNER_WAIT_TIME = 60;

	self.current_state = NULL_STATE;
	self.corner_moving_towards = 1;
	
	self.corner_wait_timer = CORNER_WAIT_TIME;
	
	self.remaining_birds = 5;
	
	self.bird_emission_timer	= 100;
	
	self.beam_out_timer = 100;

   SetAnimationSpeed(self.cppPointer, 3);


end

-----------------------------------------------------------------------------------------------------

function Circle:PrepareForWaveStart ( )
		self:SelectNewCorner();
		self.bird_emission_timer = GenerateRandomNumber(100, 200);
		self.remaining_birds = 5;
--		self.remaining_birds = 1;

      local newX = 0;
      local newY = 0;

		local starting_side = GenerateRandomNumber(0,3);
		if		(  starting_side == 0 ) then
			-- top side
			newX = GenerateRandomNumber(self:GetMinXForPlacement(),self:GetMaxXForPlacement());
			newY = self:GetMinYForPlacement();
		elseif	(  starting_side == 1 ) then
			-- right side
			newX = self:GetMaxXForPlacement();
			newY = GenerateRandomNumber(self:GetMinYForPlacement(),self:GetMaxYForPlacement() );
			
		elseif	(  starting_side == 2 ) then
			-- bottom
			newX = GenerateRandomNumber(self:GetMinXForPlacement(),self:GetMaxXForPlacement());
			newY = self:GetMaxYForPlacement();
		
		
		elseif	(  starting_side == 3 ) then		
			-- left
			newX = self:GetMinXForPlacement();
			newY = GenerateRandomNumber(self:GetMinYForPlacement(),self:GetMaxYForPlacement() );
		end

		SetLuaGameObjectXandY( self.cppPointer, newX, newY );					

end
-----------------------------------------------------------------------------------------------------
function Circle:PreWaveStartUpdate( countdownTimer)
		if countdownTimer <= 70 then
         AnimateLuaGameObject( self.cppPointer );
			DrawLuaGameObject( self.cppPointer );
		end;
end;

-----------------------------------------------------------------------------------------------------
function Circle:AddNewBird()

	local circleX, circleY  = GetLuaGameObjectXAndY(self.cppPointer);

   local newBirdCppPointer = AddGameObject( BIRD_TYPE, circleX, circleY );	
   PlaySound( BIRD_EMITTED_SOUND );

   local dummyBird = {};

   

   dummyBird.x, dummyBird.y = GetLuaGameObjectXAndY(newBirdCppPointer);
   dummyBird.collisionBoxWidth, dummyBird.collisionBoxHeight = GetLuaGameObjectCollisionBoxInfo( newBirdCppPointer );
	dummyBird.playfieldMinX, dummyBird.playfieldMaxX, dummyBird.playfieldMinY, dummyBird.playfieldMaxY = GetPlayfieldLimits();

	while(WasCollisionWithUpperWall(dummyBird) 	) do
       dummyBird.y = dummyBird.y + 1;
   end;

	while(WasCollisionWithLowerWall(dummyBird) 	) do
       dummyBird.y = dummyBird.y - 1;   
   end;

	while(WasCollisionWithLeftWall(dummyBird) 	) do
       dummyBird.x = dummyBird.x + 1;   
   end;

	while(WasCollisionWithRightWall(dummyBird) 	) do
       dummyBird.x = dummyBird.x - 1;   
   end;


   SetLuaGameObjectXandY( newBirdCppPointer, dummyBird.x, dummyBird.y );
   

end;


-----------------------------------------------------------------------------------------------------

function Circle:Update ()
	
--		PrintStringList( "self.current_state = " .. self.current_state );
	
		if self.current_state == NULL_STATE then
			self:HandleNullState();
		end;
		
		if self.current_state == self.MOVING_TOWARDS_CORNER then
			self:HandleMovingTowardsCorner(self.cppPointer);
		end;
		
		if self.current_state == self.PAUSING_IN_CORNER then
			self:HandlePausingInCorner();
		end;

		if self.remaining_birds > 0 then
			self.bird_emission_timer = self.bird_emission_timer - 1;
			if self.bird_emission_timer <= 0 then
				self.bird_emission_timer = 10;
				self.remaining_birds = self.remaining_birds - 1;
            self:AddNewBird();
--				local circleX, circleY  = GetLuaGameObjectXAndY(self.cppPointer);
--				AddGameObject( BIRD_TYPE, circleX, circleY );
--				PlaySound( BIRD_EMITTED_SOUND);				
			end;
		else
			self.beam_out_timer = self.beam_out_timer - 1;
			if self.beam_out_timer <= 0 then
            SetLuaGameObjectIsDeadFlag( self.cppPointer,1 );
			end;
		end;		
	
		AnimateLuaGameObject( self.cppPointer );
		DrawLuaGameObject( self.cppPointer );
	

end
-----------------------------------------------------------------------------------------------------	



---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------

function Circle_Create(aCppPointer)
   temp = Circle:new(aCppPointer);
   circle_objects[aCppPointer] = temp;
end;
---------------------------------------------------------------------------

function Circle_Destroy(aCppPointer)
   circle_objects[aCppPointer] = nil;
end;
---------------------------------------------------------------------------


function Circle_InitializeGameObject(aCppPointer)
   theCircle = circle_objects[aCppPointer];
   theCircle:InitializeGameObject();
end;

---------------------------------------------------------------------------

function Circle_PrepareForWaveStart(aCppPointer)
   theCircle = circle_objects[aCppPointer];
   theCircle:PrepareForWaveStart();
end;

---------------------------------------------------------------------------

function Circle_PreWaveStartUpdate(aCppPointer, countdownTimer)
--   PrintStringList("Calling Circle_PreWaveStartUpdate");
   theCircle = circle_objects[aCppPointer];
   theCircle:PreWaveStartUpdate(countdownTimer);
end;

---------------------------------------------------------------------------
function Circle_Update(aCppPointer )
--   PrintStringList("Calling Circle_Update");
   theCircle = circle_objects[aCppPointer];
   theCircle:Update(countdownTimer);
end;


